@@ -25,7 +25,7 @@
 #include "dviOps.h"
 
 #ifndef lint
-static char rcsid[] VAR_UNUSED = "$Id: dviFile.c,v 1.1.1.1 1999/06/10 12:36:50 lingnau Exp $";
+static char rcsid[] VAR_UNUSED = "$Id: dviFile.c,v 1.3 1999/06/18 10:11:29 lingnau Exp $";
 #endif /* lint */
 
 static Dvi_File *dviFileList = (Dvi_File *)0;
@@ -157,28 +157,28 @@ OpenDviFile (interp, name)
      * Check whether file is already open. If so, increment reference
      * count and return pointer to the already-opened file.
      */
-
+    
     for (dviFileInfo = dviFileInfoList;
-         dviFileInfo != (Dvi_FileInfo *)0
+	 dviFileInfo != (Dvi_FileInfo *)0
 	     && (statBuf.st_dev != dviFileInfo->devNo
 		 || (statBuf.st_dev == dviFileInfo->devNo
 		     && statBuf.st_ino != dviFileInfo->inodeNo));
 	 dviFileInfo = dviFileInfo->nextPtr)
-        ;
+	;
     if (dviFileInfo != (Dvi_FileInfo *)0) {
-        dviFileInfo->refCount++;
-        return dviFileInfo;
+	dviFileInfo->refCount++;
+	return dviFileInfo;
     }
-
+	
     /*
      * Allocate and fill in a Dvi_File structure for the file.
      */
-
+    
     if ((dviFileInfo = ckalloc(sizeof(Dvi_FileInfo))) == 0) {
 	if (interp) {
 	    Tcl_SetResult(interp, "not enough memory", TCL_STATIC);
 	}
-        return (Dvi_FileInfo *)0;
+	return (Dvi_FileInfo *)0;
     }
 
     dviFileInfo->fileDesc = fileDesc;
@@ -283,7 +283,7 @@ OpenDviFile (interp, name)
     dviFileInfo->name = DviSaveStr(name);
     dviFileInfo->nextPtr = dviFileInfoList;
     dviFileInfoList = dviFileInfo;
-    
+
     return dviFileInfo;
 }
 
@@ -297,6 +297,7 @@ Dvi_ReloadFile (interp, dviFile)
     Dvi_File *listPtr;
 
     char *name = DviSaveStr(oldDviFileInfo->name);
+    unsigned int oldGeneration = oldDviFileInfo->generation;
     CloseDviFile(oldDviFileInfo, 1);
     newDviFileInfo = OpenDviFile(interp, name);
     ckfree(name);
@@ -305,9 +306,13 @@ Dvi_ReloadFile (interp, dviFile)
 	return 0;
     }
 
+    newDviFileInfo->generation = oldGeneration + 1;
+    newDviFileInfo->refCount = 0; /* will be reconstructed shortly */
+
     for (listPtr = dviFileList; listPtr; listPtr = listPtr->nextPtr) {
 	if (listPtr->infoPtr == oldDviFileInfo) {
 	    listPtr->infoPtr = newDviFileInfo;
+	    newDviFileInfo->refCount++;
 	    if (listPtr->reloadProc) {
 		(* listPtr->reloadProc)(listPtr->clientData, listPtr);
 	    }
