@@ -14,14 +14,12 @@
 #include "dviOps.h"
 
 #ifndef lint
-static char rcsid[] VAR_UNUSED = "$Id: dviInterp.c,v 1.1.1.1 1999/06/10 12:36:50 lingnau Exp $";
+static char rcsid[] VAR_UNUSED = "$Id: dviInterp.c,v 1.2 1999/06/13 23:22:27 lingnau Exp $";
 #endif /* lint */
 
 #define STD_MAXDRIFT 2		/* standard maximum deviation between true */
                                 /* and rounded pixel positions */
 
-static U8 *ProcessFontDef _ANSI_ARGS_((Dvi_Interp *dviInterp, S32 fontNum,
-                                       U8 *fontDefPtr, int inPostamble));
 static U8 *GetFirstParameter _ANSI_ARGS_((int c, U8 *codePtr, S32 *valuePtr,
                                           Dvi_StackFrame *contextPtr));
 
@@ -29,8 +27,9 @@ static void stdGlyph _ANSI_ARGS_((ClientData, Dvi_Interp *, S32, S32,
                                   Dvi_Font *, S32, S32 *, S32 *));
 static void stdRule _ANSI_ARGS_((ClientData, Dvi_Interp *, S32, S32,
                                  S32, S32));
-static int stdFontDef _ANSI_ARGS_((ClientData, Dvi_Interp *, S32, U32, U32,
-                                   U32, size_t, char *, int));
+static int stdFontDef _ANSI_ARGS_((ClientData, Dvi_Interp *, Dvi_FontList **,
+				   S32, U32, U32, U32, size_t, char *,
+				   Dvi_FontDefMode));
 static int stdSpecial _ANSI_ARGS_((ClientData, Dvi_Interp *, S32, S32,
                                    U32, char *));
 
@@ -72,7 +71,7 @@ Dvi_CreateInterp(interp, xRes, yRes, stackSize, numerator, denominator,
     }
 
     /*
-     * Allocate memory for the interpreter and stack.
+     * Allocate memory for the interpreter and stacks.
      */
 
     dviInterp = ckalloc(sizeof(Dvi_Interp));
@@ -80,12 +79,16 @@ Dvi_CreateInterp(interp, xRes, yRes, stackSize, numerator, denominator,
         return (Dvi_Interp *)0;
     }
 
-    dviInterp->stack = ckalloc(stackSize * sizeof(Dvi_StackFrame));
+    /*
+     * Allocate the stack with a reserve for virtual fonts.
+     */
+
+    dviInterp->stack = ckalloc((stackSize + 10) * sizeof(Dvi_StackFrame));
     if (dviInterp->stack == 0) {
         ckfree(dviInterp);
         return (Dvi_Interp *)0;
     }
-    dviInterp->stackSize = stackSize;
+    dviInterp->stackSize = stackSize + 10;
 
     /*
      * Initialize the various fields.
@@ -294,7 +297,8 @@ Dvi_FontsFromPostamble(dviInterp, dviFile)
     code = dviFile->infoPtr->postamble + 29;
     while (*code >= D_FNTDEF1 && *code <= D_FNTDEF4) {
         code = GetFirstParameter(*code, code+1, &fontNum, (Dvi_StackFrame *)0);
-        code = ProcessFontDef(dviInterp, fontNum, code, 1);
+        code = Dvi_ProcessFontDef(dviInterp, &dviInterp->fonts, fontNum,
+				  code, DVI_FMODE_POST);
     }
     return TCL_OK;
 }
@@ -302,7 +306,7 @@ Dvi_FontsFromPostamble(dviInterp, dviFile)
 /*
  * ------------------------------------------------------------------------
  *
- * ProcessFontDef --
+ * Dvi_ProcessFontDef --
  *
  *      Handle a single font definition (in or outside postamble). This
  *      code assumes that the first font definition parameter (the font
@@ -319,19 +323,18 @@ Dvi_FontsFromPostamble(dviInterp, dviFile)
  * ------------------------------------------------------------------------
  */
 
-static U8 *
-ProcessFontDef (dviInterp, fontNum, fontDefPtr, inPostamble)
+U8 *
+Dvi_ProcessFontDef (dviInterp, fontListPtr, fontNum, fontDefPtr, mode)
     Dvi_Interp *dviInterp;
+    Dvi_FontList **fontListPtr;
     S32 fontNum;
     U8 *fontDefPtr;
-    int inPostamble;
+    Dvi_FontDefMode mode;
 {
     U32 check;                  /* Font checksum */
     U32 fontScale;              /* Font scaling factor */
     U32 designSize;             /* Font 'design size' */
     unsigned int nameLen;       /* Length of font name */
-    Dvi_FontList *listPtr;
-    int found;			/* 1 iff font found in dviInterp's list */
 
     /*
      * Obtain font parameters from DVI file.
@@ -341,27 +344,12 @@ ProcessFontDef (dviInterp, fontNum, fontDefPtr, inPostamble)
     fontScale = DviGetU32(fontDefPtr + 4);
     designSize = DviGetU32(fontDefPtr + 8);
     nameLen = DviGetU8(fontDefPtr+12) + DviGetU8(fontDefPtr+13);
-
-    /*
-     * See whether this font has been used already in this document.
-     */
-
-    found = 0;
-    for (listPtr = dviInterp->fonts;
-         !found && listPtr != (Dvi_FontList *)0;
-         listPtr = listPtr->nextPtr) {
-        found = (fontNum == listPtr->fontNum);
-    }
-
-    /*
-     * Load the font if it was not found.
-     */
-
-    if (!found) {
-        (dviInterp->fontDefProc)(dviInterp->procData,
-                                 dviInterp, fontNum, check, fontScale,
-                                 designSize, (size_t)nameLen,
-                                 (char *)(fontDefPtr + 14), inPostamble);
+	
+    if (mode != DVI_FMODE_DVI) {
+	(dviInterp->fontDefProc)(dviInterp->procData,
+				 dviInterp, fontListPtr, fontNum, check,
+				 fontScale, designSize, (size_t)nameLen,
+				 (char *)(fontDefPtr + 14), mode);
     }
     return fontDefPtr + nameLen + 14;
 }
@@ -374,6 +362,7 @@ ProcessFontDef (dviInterp, fontNum, fontDefPtr, inPostamble)
  *      Render a piece of DVI code in the context of *dviInterp.
  *
  * Results:
+ *      A standard Tcl result.
  *
  * Side effects:
  *      The rendering procedures defined in the DVI interpreter are
@@ -397,23 +386,56 @@ Dvi_Interpret(dviInterp, code)
     Dvi_Interp *dviInterp;
     U8 *code;
 {
-    Dvi_Font *f = 0;
-    S32 fontSpaceLeft, fontSpaceRight, fontSpaceVert;
-    U32 maxDrift;               /* cache value from dviInterp */
-    int done = 0;
-    U8 *codeStart = code;
-
     if (code == (U8 *)0) {
         return TCL_ERROR;
     }
 
+    return Dvi_DoInterpret(dviInterp, code, (Dvi_Font *)0, dviInterp->fonts);
+}
+
+/*
+ * ------------------------------------------------------------------------
+ *
+ * Dvi_DoInterpret --
+ *
+ *      Render a piece of DVI code in the context of *dviInterp. This
+ *      procedure does the actual work; it calls itself recursively
+ *      to deal with characters from virtual fonts.
+ *
+ * Results:
+ *      A standard Tcl result.
+ *
+ * Side effects:
+ *      The rendering procedures defined in the DVI interpreter are
+ *      called at appropriate moments during the interpretation.
+ *
+ * ------------------------------------------------------------------------
+ */
+
+int
+Dvi_DoInterpret(dviInterp, code, f, fontListPtr)
+    Dvi_Interp *dviInterp;
+    U8 *code;
+    Dvi_Font *f;
+    Dvi_FontList *fontListPtr;
+{
+    S32 fontSpaceLeft, fontSpaceRight, fontSpaceVert;
+    U32 maxDrift;               /* cache value from dviInterp */
+    U8 *codeStart = code;
+
     maxDrift = dviInterp->maxDrift;
-    fontSpaceLeft = fontSpaceRight = fontSpaceVert = 0;
-    while (!done) {
+    if (f == (Dvi_Font *)0) {
+	fontSpaceLeft = fontSpaceRight = fontSpaceVert = 0;
+    } else {
+	fontSpaceRight = f->fontScale / 6; /* see dvitype.web 63 */
+	fontSpaceLeft = -4 * fontSpaceRight;
+	fontSpaceVert = 5 * fontSpaceRight;
+    }
+    while (*code != D_EOP) {
         U8 o;                   /* DVI command currently being executed */
         S32 p, q;               /* Parameters for current DVI command */
 
-        o = *code++;
+	o = *code++;
         code = GetFirstParameter(o, code, &p, CONTEXT);
         switch (o) {
             case D_SET1: case D_SET2: case D_SET3: case D_SET4:
@@ -431,9 +453,41 @@ Dvi_Interpret(dviInterp, code)
             
             {
                 S32 pixelX;	/* Width of character in device pixels */
-                
-                (* dviInterp->glyphProc)(dviInterp->procData, dviInterp,
-                                         HH, VV, f, p, &q, &pixelX);
+
+		/*
+		 * If the current font is a virtual font we do not call
+		 * the current glyphProc but arrange for the virtual
+		 * character's DVI snippet to be executed.
+		 *
+		 * The case where a character is simply replaced with
+		 * a single character from the default font could be
+		 * improved to remove the PUSH and POP operations.
+		 */
+
+		if (f->type == dvi_font_vf) {
+		    U8 *codePtr;
+		    Dvi_Font *firstFont;
+		    Dvi_FontList *fontListPtr;
+
+		    if (dviInterp->stackPtr == dviInterp->stackSize) {
+			/*
+			 * Dynamically enlarge the DVI stack
+			 */
+		    }
+		    CONTEXT[1] = CONTEXT[0];
+		    dviInterp->stackPtr++;
+		    
+		    if (Dvi_GetVFInfo(f, p, &q, &pixelX, &firstFont, &codePtr,
+				      &fontListPtr) == TCL_OK) {
+			Dvi_DoInterpret(dviInterp, codePtr, firstFont,
+					fontListPtr);
+		    }
+    		    dviInterp->stackPtr--;
+		} else {
+		    (* dviInterp->glyphProc)(dviInterp->procData, dviInterp,
+					     HH, VV, f, p, &q, &pixelX);
+		}
+
                 if (o >= D_PUT1) {
                     break;
                 }
@@ -562,8 +616,8 @@ Dvi_Interpret(dviInterp, code)
                 for (listPtr = dviInterp->fonts;
                      listPtr != (Dvi_FontList *)0 && p != listPtr->fontNum;
                      listPtr = listPtr->nextPtr)
-                    ;
-                
+		    ;
+
                 if (listPtr != (Dvi_FontList *)0) {
                     f = listPtr->fontPtr;
                     if (f == (Dvi_Font *)0) {
@@ -592,7 +646,8 @@ Dvi_Interpret(dviInterp, code)
 	     */
 
             case D_FNTDEF1: case D_FNTDEF2: case D_FNTDEF3: case D_FNTDEF4:
-                code = ProcessFontDef(dviInterp, p, code, 0);
+                code = Dvi_ProcessFontDef(dviInterp, &dviInterp->fonts,
+					  p, code, DVI_FMODE_DVI);
                 break;
 
             case D_NOP:
@@ -603,7 +658,7 @@ Dvi_Interpret(dviInterp, code)
                 break;
 
             case D_EOP:
-                done = 1;
+                /* done = 1; */
                 break;
 
             case D_PUSH:
@@ -767,7 +822,7 @@ stdGlyph (clientData, dviInterp, x, y, fontPtr, character, tfmWidthPtr,
     if (output != 0) {
         Tcl_DStringAppend(output, buffer, -1);
     }
-#if 0
+#if 1
     fputs(buffer, stderr);
 #endif
 }
@@ -793,26 +848,28 @@ stdRule (clientData, dviInterp, x, y, height, width)
 }
 
 static int
-stdFontDef (clientData, dviInterp, font, check, fontScale, designSize,
-	    nameLen, name, inPostamble)
+stdFontDef (clientData, dviInterp, fontListPtr, fontNum,
+	    check, fontScale, designSize, nameLen, name, mode)
     ClientData clientData;
     Dvi_Interp *dviInterp;
-    S32 font;
+    Dvi_FontList **fontListPtr;
+    S32 fontNum;
     U32 check;
     U32 fontScale;
     U32 designSize;
     size_t nameLen;
     char *name;
-    int inPostamble;
+    Dvi_FontDefMode mode;
 {
     Tcl_DString *output = (Tcl_DString *)clientData;
     char buffer[1024];
     
-    Dvi_FontAdd(dviInterp, font, check, fontScale, designSize, nameLen, name);
-    sprintf(buffer, "FONTDEF %ld %lu %lu %lu %.*s\n", (long)font,
+    sprintf(buffer, "FONTDEF %ld %lu %lu %lu %.*s\n", (long)fontNum,
             (unsigned long)check, (unsigned long)fontScale,
             (unsigned long)designSize, (int)nameLen, name);
-    if (!inPostamble && output != 0) {
+    Dvi_FontAdd(dviInterp, fontListPtr, fontNum,
+		check, fontScale, designSize, nameLen, name);
+    if (mode == DVI_FMODE_DVI && output != 0) {
         Tcl_DStringAppend(output, buffer, -1);
     }
 #if 0
