@@ -14,7 +14,7 @@
 #include "dviInt.h"
 
 #ifndef lint
-static char rcsid[] VAR_UNUSED = "$Id: font.c,v 1.1.1.1 1999/06/10 12:36:50 lingnau Exp $";
+static char rcsid[] VAR_UNUSED = "$Id: font.c,v 1.2 1999/06/13 23:22:27 lingnau Exp $";
 #endif /* lint */
 
 /*
@@ -94,8 +94,10 @@ Dvi_CreateFontType (type, name, loadProc, glyphProc, closeProc)
  */
 
 int
-Dvi_FontAdd(dviInterp, fontNum, check, fontScale, designSize, nameLen, name)
+Dvi_FontAdd(dviInterp, fontListPtr, fontNum, check, fontScale, designSize,
+	    nameLen, name)
     Dvi_Interp *dviInterp;      /* Interpreter for adding font to */
+    Dvi_FontList **fontListPtr;	/* Address of font list */
     S32 fontNum;                /* Number of font to be installed */
     U32 check;                  /* Font checksum from DVI file */
     U32 fontScale;              /* Scaling factor from DVI file */
@@ -105,7 +107,7 @@ Dvi_FontAdd(dviInterp, fontNum, check, fontScale, designSize, nameLen, name)
 {
     Dvi_FontList *newItemPtr;
     Dvi_Font *fontPtr;
-    
+
     fontPtr = Dvi_FontFind(dviInterp, check, fontScale,
                            designSize, nameLen, name);
     
@@ -119,11 +121,11 @@ Dvi_FontAdd(dviInterp, fontNum, check, fontScale, designSize, nameLen, name)
         Dvi_FontFree(fontPtr);
         return TCL_ERROR;
     }
-    
+
     newItemPtr->fontNum = fontNum;
     newItemPtr->fontPtr = fontPtr;
-    newItemPtr->nextPtr = dviInterp->fonts;
-    dviInterp->fonts = newItemPtr;
+    newItemPtr->nextPtr = *fontListPtr;
+    *fontListPtr = newItemPtr;
     
     return TCL_OK;
 }
