@@ -11,6 +11,7 @@
 #if ENABLE_TK
 
 #include <math.h>
+#include <stdlib.h>
 #include <string.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -20,7 +21,7 @@
 #include "dviInt.h"
 
 #ifndef lint
-static char rcsid[] VAR_UNUSED = "$Id: image.c,v 1.1.1.1 1999/06/10 12:36:50 lingnau Exp $";
+static char rcsid[] VAR_UNUSED = "$Id: image.c,v 1.3 1999/06/15 16:21:10 lingnau Exp $";
 #endif /* lint */
 
 #ifndef MICKEY_MOUSE
@@ -57,6 +58,7 @@ typedef struct DviMaster {
     int validatePageNum;	/* whether requests for non-existent pages
 				   are errors */
     unsigned int pageNum;	/* Current (absolute) page number */
+    unsigned long generation;	/* Incremented when file is reloaded */
     Tk_Uid fileString;		/* DVI file cookie for the image */
     Tk_Uid fgUid;		/* Value of -foreground option (malloc'ed) */
     Tk_Uid bgUid;		/* Value of -background option (malloc'ed) */
@@ -195,8 +197,9 @@ static void RenderGlyph _ANSI_ARGS_((ClientData clientData,
 				     Dvi_Interp *dviInterp, S32 x, S32 y,
 				     Dvi_Font *fontPtr, S32 character,
 				     S32 *tfmWidthPtr, S32 *pixelWidthPtr));
-static int RenderFontDef _ANSI_ARGS_((ClientData, Dvi_Interp *, S32, U32, U32,
-				      U32, unsigned int, char *, int));
+static int RenderFontDef _ANSI_ARGS_((ClientData, Dvi_Interp *,
+				      Dvi_FontList **, S32, U32, U32,
+				      U32, size_t, char *, Dvi_FontDefMode));
 
 /*
  * ------------------------------------------------------------------------
@@ -274,10 +277,7 @@ DviImgCreate(interp, name, argc, argv, typePtr, master, clientDataPtr)
     masterPtr->validatePageNum = 0;
     masterPtr->maxDrift = 2;
     masterPtr->fileString = (char *)0;
-#if 0
-    masterPtr->dviFile = (Dvi_File *)0;
-    masterPtr->dviCodePtr = (U8 *)0;
-#endif
+    masterPtr->generation = 0;
     masterPtr->specialCommand = (char *)0;
     masterPtr->instancePtr = (DviInstance *)0;
 
@@ -360,10 +360,12 @@ DviImgConfigureMaster(masterPtr, argc, argv, flags)
 		 == (Dvi_File *)0) {
 		return TCL_ERROR;
 	    }
+	    masterPtr->generation = dviFile->infoPtr->generation;
 	    Dvi_SetResolution(masterPtr->dviInterp, masterPtr->resolutionX,
 			      masterPtr->resolutionY, dviFile->infoPtr->num,
 			      dviFile->infoPtr->den, dviFile->infoPtr->mag);
 #if !MICKEY_MOUSE
+	    masterPtr->dviInterp->fontDefProc = RenderFontDef;
 	    if (Dvi_FontsFromPostamble(masterPtr->dviInterp, dviFile)
 		    != TCL_OK) {
 		Dvi_CloseFile(dviFile);
@@ -374,10 +376,6 @@ DviImgConfigureMaster(masterPtr, argc, argv, flags)
 	} else {
 	    dviFile = (Dvi_File *)0;
 	}
-#if NO_DVIFILE
-	masterPtr->dviInterp->dviFile = dviFile;
-	masterPtr->dviFile = dviFile;
-#endif
     } else {
 	Dvi_SetResolution(masterPtr->dviInterp, masterPtr->resolutionX,
 			  masterPtr->resolutionY, 25400000, 473628672, 1000);
@@ -420,7 +418,6 @@ DviImgConfigureMaster(masterPtr, argc, argv, flags)
 	int dimenc;
 	char **dimenv;
 
-	fprintf(stderr, "user-def paper size\n");
 	if (Tcl_SplitList(masterPtr->interp, masterPtr->size,
 			  &dimenc, &dimenv) != TCL_OK) {
 	    return TCL_ERROR;
@@ -765,18 +762,23 @@ DviImgCmd(clientData, interp, argc, argv)
 		return TCL_ERROR;
 	    }
 	    masterPtr->render = 1;
-#if NO_DVIFILE
-	    dviCodePtr = Dvi_FindPage(masterPtr->dviFile, &pageSpec,
-				      &pageNum);
-#else
 	    if ((dviFile = Dvi_GetFileByCookie(masterPtr->interp,
 					       masterPtr->fileString,
 					       TCL_LEAVE_ERR_MSG))
 		== (Dvi_File *)0) {
 		return TCL_ERROR;
 	    }
+	    if (dviFile->infoPtr->generation > masterPtr->generation) {
+		Dvi_ResetInterp(masterPtr->dviInterp, 1);
+		if (Dvi_FontsFromPostamble(masterPtr->dviInterp, dviFile)
+		    != TCL_OK) {
+		    masterPtr->render = 0;
+		    Tcl_SetResult(masterPtr->interp,
+				  "couldn't re-read fonts", TCL_STATIC);
+		    return TCL_ERROR;
+		}
+	    }
 	    dviCodePtr = Dvi_FindPage(dviFile, &pageSpec, &pageNum);
-#endif
 	    if (dviCodePtr == (U8 *)0) {
 		if (masterPtr->validatePageNum) {
 		    Tcl_AppendResult(interp, "Couldn't find page \"", argv[2],
@@ -786,7 +788,6 @@ DviImgCmd(clientData, interp, argc, argv)
 		    masterPtr->render = 0;
 		}
 	    }
-/*	    masterPtr->dviCodePtr = dviCodePtr + 45;*/
 	    masterPtr->pageNum = pageNum;
 	    masterPtr->firstTimePage = 1;
 	    Tk_ImageChanged(masterPtr->tkMaster, 0, 0, masterPtr->width,
@@ -1287,23 +1288,24 @@ RenderRule (clientData, dviInterp, x, y, width, height)
 }
 
 static int
-RenderFontDef (clientData, dviInterp, font, check, fontScale, designSize,
-	    nameLen, name, inPostamble)
+RenderFontDef (clientData, dviInterp, fontListPtr, fontNum,
+	       check, fontScale, designSize, nameLen, name, mode)
     ClientData clientData;
     Dvi_Interp *dviInterp;
-    S32 font;
+    Dvi_FontList **fontListPtr;
+    S32 fontNum;
     U32 check;
     U32 fontScale;
     U32 designSize;
-    unsigned int nameLen;
+    size_t nameLen;
     char *name;
-    int inPostamble;
+    Dvi_FontDefMode mode;
 {
-    fprintf(stderr, "font %ld: %*.*s\n", (long int)font,
-	    (int)nameLen, (int)nameLen, name);
-    if (inPostamble) {
-	Dvi_FontAdd(dviInterp, font, check, fontScale, designSize,
-		    nameLen, name);
+    /* fprintf(stderr, "font %ld: %*.*s mode=%d\n",
+       (long int)fontNum, (int)nameLen, (int)nameLen, name, mode);*/
+    if (mode != DVI_FMODE_DVI) {
+	Dvi_FontAdd(dviInterp, fontListPtr, fontNum,
+		    check, fontScale, designSize, nameLen, name);
     }
     return TCL_OK;
 }
