@@ -12,11 +12,11 @@
  */
 
 #include <string.h>
-#include "kpathsea.h"
+#include "kpathsea/kpathsea.h"
 #include "dviInt.h"
 
 #ifndef lint
-static char rcsid[] VAR_UNUSED = "$Id: init.c,v 1.1.1.1 1999/06/10 12:36:50 lingnau Exp $";
+static char rcsid[] VAR_UNUSED = "$Id: init.c,v 1.3 1999/06/13 23:22:27 lingnau Exp $";
 #endif /* lint */
 
 typedef struct DviFileCookie {
@@ -1272,6 +1272,7 @@ Dvi_Init (interp)
     int resolution;
 
     Dvi_CreateFontType_PK();
+    Dvi_CreateFontType_VF();
     Dvi_CreateFontType_TFM();
 
     if (Tcl_PkgProvide(interp, DVI_NAME, DVI_VERSION) != TCL_OK) {
