@@ -1,9 +1,9 @@
 # tkdvi.tcl -- Basic setup for TkDVI.
 # Copyright  1999 Anselm Lingnau <lingnau@tm.informatik.uni-frankfurt.de>.
 # See file COPYING for conditions on use and distribution.
-# $Id: tkdvi.tcl,v 1.1.1.1 1999/06/10 12:36:50 lingnau Exp $
+# $Id: tkdvi.tcl,v 1.2 1999/06/16 07:46:40 lingnau Exp $
 
-package provide tkdvi 0.1
+package provide tkdvi 0.2
 
 # The DVI package can be configured at load time by setting the
 # following variables prior to the `package require dvi'. The values
@@ -18,8 +18,13 @@ package provide tkdvi 0.1
 package require dvi
 
 namespace eval tkdvi {
-    proc version {} {
-	return "0.1 (Alpha test)"
+    variable version "Unspecified"
+    proc version {{ver {}}} {
+	variable version
+	if {[string compare $ver {}] != 0} {
+	    set version $ver
+	}
+	return $version
     }
     proc reloadAll {} {
 	foreach w [winfo children .] {
